﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.IO;
using System.Runtime.Serialization;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Persistence.Default.Xml;

namespace HeuristicLab.Hive.Contracts.BusinessObjects {
  [StorableClass]
  [DataContract]
  [Serializable]
  public class SerializedJob : IDeepCloneable {
    [Storable]
    [DataMember]
    public JobDto JobInfo { get; set; }
    [Storable]
    [DataMember]
    public byte[] SerializedJobData { get; set; }

    public SerializedJob() { }
    [StorableConstructor]
    protected SerializedJob(bool deserializing) { }
    protected SerializedJob(SerializedJob original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      this.JobInfo = cloner.Clone(original.JobInfo);
      this.SerializedJobData = new byte[this.SerializedJobData.Length];
      Array.Copy(original.SerializedJobData, this.SerializedJobData, original.SerializedJobData.Length);
    }
    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new SerializedJob(this, cloner);
    }
    public object Clone() {
      return Clone(new Cloner());
    }

    public static byte[] Serialize(object obj) {
      MemoryStream memStream = new MemoryStream();
      XmlGenerator.Serialize(obj, memStream);
      byte[] jobByteArray = memStream.ToArray();
      memStream.Dispose();
      return jobByteArray;
    }

    public static T Deserialize<T>(byte[] sjob) {
      MemoryStream memStream = new MemoryStream(sjob);
      T job = XmlParser.Deserialize<T>(memStream);
      memStream.Dispose();
      return job;
    }
  }
}